@@ -491,7 +491,7 @@ parser.add_option_group(other)
 # Default values
 
 parser.set_defaults(
-    ignore_dir=['.svn', 'CVS', '{arch}', '.arch-ids', '_darcs'],
+    ignore_dir=['.git', '.svn', 'CVS', '{arch}', '.arch-ids', '_darcs'],
     tests_pattern='^tests$',
     at_level=1,
     test_file_pattern='^test',
@@ -559,7 +559,7 @@ def get_options(args=None, defaults=None):
 
     if options.showversion:
         dist = pkg_resources.require('zope.testrunner')[0]
-        print('zope.app.testrunner version %s' % dist.version)
+        print('zope.testrunner version %s' % dist.version)
         options.fail = True
         return options
 
@@ -624,6 +624,12 @@ def get_options(args=None, defaults=None):
 
     options.prefix = [(path + os.path.sep, package)
                       for (path, package) in options.test_path]
+
+    # Sort prefixes so that longest prefixes come first.
+    # That is because only first match is evaluated which
+    # can be a problem with nested source packages.
+    options.prefix.sort(key=lambda p: len(p[0]), reverse=True)
+
     if options.all:
         options.at_level = sys.maxsize
 
