@@ -15,9 +15,11 @@
 """
 from __future__ import print_function
 
+import collections
 import subprocess
 import errno
 import gc
+import inspect
 import re
 import sys
 import threading
@@ -31,8 +33,11 @@ except ImportError:
 from six import StringIO
 from zope.testrunner.find import import_name
 from zope.testrunner.find import name_from_layer, _layer_name_cache
-from zope.testrunner.refcount import TrackRefs
+from zope.testrunner.layer import UnitTests
+from zope.testrunner.layer import EmptyLayer
+from zope.testrunner.layer import EmptySuite
 from zope.testrunner.options import get_options
+from zope.testrunner.refcount import TrackRefs
 import zope.testrunner
 import zope.testrunner.coverage
 import zope.testrunner._doctest
@@ -122,6 +127,12 @@ class Runner(object):
         self.tests_by_layer_name = {}
 
     def ordered_layers(self):
+        if (self.options.processes > 1 and not self.options.resume_layer):
+            # if we want multiple processes, we need a fake layer as first
+            # to start spreading out layers/tests to subprocesses
+            # but only if this is not in the subprocess
+            yield (name_from_layer(EmptyLayer), EmptyLayer, EmptySuite())
+
         layer_names = dict([(layer_from_name(layer_name), layer_name)
                             for layer_name in self.tests_by_layer_name])
         for layer in order_by_bases(layer_names):
@@ -421,7 +432,7 @@ class SetUpLayerFailure(unittest.TestCase):
         pass
 
     def __str__(self):
-        return "Layer: %s.%s" % (self.layer.__module__, self.layer.__name__)
+        return "Layer: %s" % (name_from_layer(self.layer))
 
 
 
@@ -880,14 +891,42 @@ def layer_from_name(layer_name):
                              % (module_name, module_layer_name))
 
 
-def order_by_bases(layers):
-    """Order the layers from least to most specific (bottom to top)
+def layer_sort_key(layer):
+    """Compute sort key for layers.
+
+    Based on the reverse MRO ordering in order to put layers with shared base
+    layers next to each other.
     """
-    named_layers = [(name_from_layer(layer), layer) for layer in layers]
-    named_layers.sort()
-    named_layers.reverse()
+    seen = set([])
+    key = []
+
+    # Note: we cannot reuse gather_layers() here because it uses a
+    # different traversal order.
+    def _gather(layer):
+        seen.add(layer)
+        # We make the simplifying assumption that the order of initialization
+        # of base layers does not matter.  Given that, traversing the bases
+        # in reverse order here keeps the ordering of layers in
+        # testrunner-layers.txt the same as it was in older versions of
+        # zope.testrunner, so let's use that.
+        for base in layer.__bases__[::-1]:
+            if base is not object and base not in seen:
+                _gather(base)
+        key.append(layer)
+
+    _gather(layer)
+    return tuple(name_from_layer(l) for l in key if l != UnitTests)
+
+
+def order_by_bases(layers):
+    """Order the layers from least to most specific (bottom to top).
+
+    Puts unit tests first.  Groups layers with common base layers together.
+    Sorts the rest alphabetically.  Removes duplicates.
+    """
+    layers = sorted(layers, key=layer_sort_key, reverse=True)
     gathered = []
-    for name, layer in named_layers:
+    for layer in layers:
         gather_layers(layer, gathered)
     gathered.reverse()
     seen = {}
